local _, A = ...

A.currentpathID = nil

A.matches = {}
A.matchIndex = 1

local EventFrame = CreateFrame("Frame")
EventFrame:RegisterEvent("TAXIMAP_OPENED")
EventFrame:RegisterEvent("TAXIMAP_CLOSED")
EventFrame:SetScript("OnEvent", function(self, event, ...)
  return self[event] and self[event](self, ...)
end)


function EventFrame:TAXIMAP_OPENED()
  A.nodes = C_TaxiMap.GetAllTaxiNodes(GetTaxiMapID())

  if not FlightMapFrameSearchBox then A:CreateFrames() end

  A.FPSearchbox:SetFocus()
end


function A:CreateFrames()


  A.FPSearchbox = CreateFrame("EditBox", "FlightMapFrameSearchBox", FlightMapFrame, "InputBoxTemplate")
  local box = A.FPSearchbox
  box:SetParent(FlightMapFrame)
  box:SetFontObject("ChatFontNormal")
  --box:SetFont("Interface\\AddOns\\FlightPathSearch\\Fonts\\PTSansNarrow.ttf", 12, "OUTLINE")
  box:SetWidth(200)
  box:SetHeight(25)
  box:SetAutoFocus(false)
  box:SetFrameStrata("DIALOG")
  box:SetPoint("BOTTOM", FlightMapFrame, "BOTTOM", 0, 10)

  A.currentPathName = A.FPSearchbox:CreateFontString(nil, "OVERLAY")
  A.currentPathName:SetFont("Interface\\AddOns\\FlightPathSearch\\Fonts\\PTSansNarrow.ttf", 14, "OUTLINE")
  A.currentPathName:SetPoint("BOTTOM", A.FPSearchbox, "TOP", 0, 5)
  A.currentPathName:SetText("Start searching...")

  box:SetScript("OnEscapePressed", function(self)
    if self:GetText() == "" then
      self:ClearFocus()
    else
      self:SetText("")
      A.matches = {}
      A.matchIndex = 1
    end
  end)

  box:SetScript("OnEnterPressed", function(self)
    if A.matches and A.matches[A.matchIndex] then
      TakeTaxiNode(A.matches[A.matchIndex].slotIndex)
      self:SetText("")
    end
  end)

  box:SetScript("OnTextChanged", function(self)
    local text = self:GetText():lower():gsub(" ", "")
    if text == "" then
      A.currentpathID = nil
      A.currentPathName:SetText("Start searching...")
      return
    end

    wipe(A.matches)
    A.matchIndex = 1

    for _, node in ipairs(A.nodes) do
      local name = node.name:lower():gsub(" ", "")
      if string.find(name, text) and node.state < 2 then
        tinsert(A.matches, node)
      end
    end

    if #A.matches == 0 then
      A.currentPathName:SetText("No results")
      return
    end

    if A.matches[A.matchIndex].state == 0 then
      A.currentPathName:SetText("This is your current location")
      FlightMapFrame:PanAndZoomTo(A.matches[A.matchIndex].position:GetXY())
      return
    end

    if #A.matches > 1 then
      A.currentPathName:SetText("[" .. A.matchIndex .. "] " .. A.matches[A.matchIndex].name)
      FlightMapFrame:PanAndZoomTo(A.matches[A.matchIndex].position:GetXY())
    else
      A.currentPathName:SetText(A.matches[A.matchIndex].name)
      FlightMapFrame:PanAndZoomTo(A.matches[A.matchIndex].position:GetXY())
    end
  end)


  box:SetScript("OnTabPressed", function(self)
    if #A.matches > 1 then
      local reverse = IsShiftKeyDown()

      if A.matchIndex == #A.matches and reverse then
        A.matchIndex = A.matchIndex - 1

      elseif A.matchIndex == #A.matches then
        A.matchIndex = 1

      elseif A.matchIndex == 1 and reverse then
        A.matchIndex = #A.matches

      elseif reverse then
        A.matchIndex = A.matchIndex - 1

      else
        A.matchIndex = A.matchIndex + 1
      end
      A.currentPathName:SetText("[" .. A.matchIndex .. "] " .. A.matches[A.matchIndex].name)
      FlightMapFrame:PanAndZoomTo(A.matches[A.matchIndex].position:GetXY())
    end
  end)
end


